/*-----------------------------------------------------------------------------
 * Name:    GLCD_SAM3U-EK.c
 * Purpose: Graphic LCD interface (240x320 pixels) for Graphic LCD with
 *          8-bit interface for SAM4S-EK evaluation board
 * Rev.:    1.00
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2013 - 2014 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include <stddef.h>
#include "sam3u.h"
#include "GLCD_Config.h"
#include "Board_GLCD.h"

/*********************** Hardware specific configuration **********************/

/*--------------- Graphic LCD interface hardware definitions -----------------*/

/* Note: LCD /CS is CCS2                                                      */
#define LCD_BASE   (0x62000000u)

#define LCD_REG16   (*((volatile uint16_t *)(LCD_BASE  )))
#define LCD_DAT16   (*((volatile uint16_t *)(LCD_BASE+2)))

/************************ Global variables ************************************/

static uint16_t   foreground_color = GLCD_COLOR_BLACK;
static uint16_t   background_color = GLCD_COLOR_WHITE;
static GLCD_FONT *active_font      = NULL;

#if (GLCD_SWAP_XY == 0)
static uint32_t   Scroll;
#endif

/************************ Local auxiliary functions ***************************/

/**
  \fn          void delay_ms (int ms)
  \brief       Delay execution for a specified number of milliseconds
  \param[in]   ms     number of while loop iterations
*/
static void delay_ms (int ms) {
  ms *= (SystemCoreClock/10000);
  while (ms--) { __nop(); __nop(); __nop(); __nop(); __nop(); __nop(); }
}



/**
  \fn          void wr_cmd (uint8_t cmd)
  \brief       Write a command to the LCD controller
  \param[in]   cmd  Command to write
*/
static __inline void wr_cmd (uint8_t cmd) {

  LCD_REG16 = cmd;
}

/**
  \fn          void wr_dat (uint16_t dat)
  \brief       Write data to the LCD controller
  \param[in]   dat  Data to write
*/
static __inline void wr_dat (uint16_t dat) {

  LCD_DAT16 = dat;
}

/**
  \fn          void wr_dat_start (void)
  \brief       Start writing data to LCD controller
*/
static __inline void wr_dat_start (void) {
  /* only used for SPI interface */
}

/**
  \fn          void wr_dat_stop (void)
  \brief       Stop writing data to LCD controller
*/
static __inline void wr_dat_stop (void) {
  /* only used for SPI interface */
}

/**
  \fn          void wr_dat_only (uint16_t dat)
  \brief       Write data to the LCD controller (without start/stop sequence)
  \param[in]   dat  Data to write
*/
static __inline void wr_dat_only (uint16_t dat) {

  LCD_DAT16 = dat;
}

#if 0
/**
  \fn          uint16_t rd_dat (void)
  \brief       Read data from the LCD controller
  \returns     Data read
*/
static __inline uint16_t rd_dat (void) {

  return (LCD_DAT16);
}
#endif

/**
  \fn          void wr_reg (uint8_t reg, uint16_t val)
  \brief       Write a value to the LCD register
  \param[in]   reg  Register to be written
  \param[in]   val  Value to write to the register
*/
static __inline void wr_reg (uint8_t reg, uint16_t val) {
  wr_cmd(reg);
  wr_dat(val);
}

#if 0
/**
  \fn          uint16_t rd_reg (uint8_t reg)
  \brief       Read a value from the LCD register
  \param[in]   reg  Register to be read
  \returns     Value read from the register
*/
static __inline uint16_t rd_reg (uint8_t reg) {
  wr_cmd(reg);
  return (rd_dat());
}
#endif


/**
  \fn          void GLCD_SetWindow (uint32_t x, uint32_t y, uint32_t w, uint32_t h)
  \brief       Set draw window region
  \param[in]   x  Horizontal position in pixel
  \param[in]   y  Vertical position in pixel
  \param[in]   w  Window width in pixel
  \param[in]   h  Window height in pixel
*/
static void GLCD_SetWindow (uint32_t x, uint32_t y, uint32_t w, uint32_t h) {
  uint32_t xe, ye;

  xe = x+w-1;
  ye = y+h-1;

  wr_reg(0x02, x  >>    8);           /* Column address start MSB           */
  wr_reg(0x03, x  &  0xFF);           /* Column address start LSB           */
  wr_reg(0x04, xe >>    8);           /* Column address end MSB             */
  wr_reg(0x05, xe &  0xFF);           /* Column address end LSB             */

  wr_reg(0x06, y  >>    8);           /* Row address start MSB              */
  wr_reg(0x07, y  &  0xFF);           /* Row address start LSB              */
  wr_reg(0x08, ye >>    8);           /* Row address end MSB                */
  wr_reg(0x09, ye &  0xFF);           /* Row address end LSB                */
}


/************************ Exported functions **********************************/

#define LCD_D0_D14      (0xFE01FE00)
#define LCD_D15         (PIO_PB6)
#define LCD_RS          (PIO_PB8)
#define LCD_RD          (PIO_PB19)
#define LCD_WR          (PIO_PB23)
#define LCD_NCS2        (PIO_PC16)
#define LCD_BACKLIGHT   (PIO_PC19)

/**
  \fn          int32_t GLCD_Initialize (void)
  \brief       Initialize Graphic LCD
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_Initialize (void) {
  uint32_t i;

/* Configure the LCD Control pins --------------------------------------------*/
  PMC->PMC_WPMR = 0x504D4300u;                  /* Disable write protect      */
  PMC->PMC_PCER0  = ((1u << ID_PIOB) |          /* Enable clock for PIOB      */
                     (1u << ID_PIOC) );         /* Enable clock for PIOC      */
  PMC->PMC_WPMR = 0x504D4301u;                  /* Enable write protect       */

/* Configure the Data Bus pins -----------------------------------------------*/
  PIOB->PIO_PDR   =  (LCD_D0_D14 | LCD_D15);    /* Enable Peripheral function */
  PIOB->PIO_IDR   =  (LCD_D0_D14 | LCD_D15);    /* Disable interrupt          */
  PIOB->PIO_ABSR &= ~(LCD_D0_D14);              /* Select Peripheral A        */
  PIOB->PIO_ABSR |=  (LCD_D15);                 /* Select Peripheral B        */

/* Configure the Control pins ------------------------------------------------*/
  PIOB->PIO_PDR   =  (LCD_RS);                  /* Enable Peripheral function */
  PIOB->PIO_IDR   =  (LCD_RS);                  /* Disable interrupt          */
  PIOB->PIO_ABSR |=  (LCD_RS);                  /* Select Peripheral B        */

  PIOB->PIO_PDR   =  (LCD_RD | LCD_WR);         /* Enable Peripheral function */
  PIOB->PIO_IDR   =  (LCD_RD | LCD_WR);         /* Disable interrupt          */
  PIOB->PIO_ABSR &= ~(LCD_RD | LCD_WR);         /* Select Peripheral A        */

  PIOC->PIO_PDR   =  (LCD_NCS2);                /* Enable Peripheral function */
  PIOC->PIO_IDR   =  (LCD_NCS2);                /* Disable interrupt          */
  PIOC->PIO_ABSR &= ~(LCD_NCS2);                /* Select Peripheral A        */

/* Configure the Backlight pins ----------------------------------------------*/
  PIOC->PIO_IDR   =                             /* Disable interrupt          */
  PIOC->PIO_OER   =                             /* Enable output              */
  PIOC->PIO_PER   = (LCD_BACKLIGHT);            /* Disable Peripheral function*/
  PIOC->PIO_CODR  = (LCD_BACKLIGHT);            /* Backlight off              */

/*-- SMC Configuration -------------------------------------------------------*/
/*--------------------------- CS2 --------------------------------------------*/
  PMC->PMC_WPMR = 0x504D4300u;                  /* Disable write protect      */
  PMC->PMC_PCER0  = (1u << ID_SMC);             /* Enable clock for SMC       */
  PMC->PMC_WPMR = 0x504D4301u;                  /* Enable write protect       */

  SMC->SMC_CS_NUMBER[2].SMC_SETUP = (SMC_SETUP_NWE_SETUP(4)    |
                                     SMC_SETUP_NCS_WR_SETUP(2) |
                                     SMC_SETUP_NRD_SETUP(4)    |
                                     SMC_SETUP_NCS_RD_SETUP(2)  );

  SMC->SMC_CS_NUMBER[2].SMC_PULSE = (SMC_PULSE_NWE_PULSE(5)    |
                                     SMC_PULSE_NCS_WR_PULSE(18)|
                                     SMC_PULSE_NRD_PULSE(5)    |
                                     SMC_PULSE_NCS_RD_PULSE(18) );

  SMC->SMC_CS_NUMBER[2].SMC_CYCLE = (SMC_CYCLE_NWE_CYCLE(12) |
                                     SMC_CYCLE_NRD_CYCLE(22)  );

  SMC->SMC_CS_NUMBER[2].SMC_MODE = ((SMC_MODE_READ_MODE)  |
                                    (SMC_MODE_WRITE_MODE) |
                                    (SMC_MODE_DBW)         );

  delay_ms(50);                         /* Delay 50 ms                        */

  /* Start internal OSC                                                       */
  wr_reg(0x19, 0x49);                   /* OSCADJ=10 0000, OSD_EN=1 // 60Hz   */
  wr_reg(0x93, 0x0C);                   /* RADJ=1100                          */

  /* Power on flow                                                            */
  wr_reg(0x44, 0x4D);                   /* VCM=100 1101                       */
  wr_reg(0x45, 0x11);                   /* VDV=1 0001                         */
  wr_reg(0x20, 0x40);                   /* BT=0100                            */
  wr_reg(0x1D, 0x07);                   /* VC1=111                            */
  wr_reg(0x1E, 0x00);                   /* VC3=000                            */
  wr_reg(0x1F, 0x04);                   /* VRH=0100                           */

  wr_reg(0x1C, 0x04);                   /* AP=100                             */
  wr_reg(0x1B, 0x10);                   /* GASENB=0, PON=1, DK=0, XDK=0, DDVDH_TRI=0, STB=0 */
  delay_ms(50);

  wr_reg(0x43, 0x80);                   /* Set VCOMG=1                        */
  delay_ms(50);

  /* Gamma for CMO 2.8                                                        */
  wr_reg(0x46, 0x95);
  wr_reg(0x47, 0x51);
  wr_reg(0x48, 0x00);
  wr_reg(0x49, 0x36);
  wr_reg(0x4A, 0x11);
  wr_reg(0x4B, 0x66);
  wr_reg(0x4C, 0x14);
  wr_reg(0x4D, 0x77);
  wr_reg(0x4E, 0x13);
  wr_reg(0x4F, 0x4C);
  wr_reg(0x50, 0x46);
  wr_reg(0x51, 0x46);

  /* Display Setting                                                          */
  wr_reg(0x01, 0x06);                   /* IDMON=0, INVON=1, NORON=1, PTLON=0 */
#if (GLCD_SWAP_XY == 1)
  wr_reg(0x16, 0x68);                   /* MY=0, MX=1, MV=1, BGR=1            */
#else
  wr_reg(0x16, 0xC8);                   /* MY=1, MX=1, MV=0, BGR=1            */
#endif
  wr_reg(0x23, 0x95);                   /* N_DC=1001 0101                     */
  wr_reg(0x24, 0x95);                   /* P_DC=1001 0101                     */
  wr_reg(0x25, 0xFF);                   /* I_DC=1111 1111                     */
  wr_reg(0x27, 0x06);                   /* N_BP=0000 0110                     */
  wr_reg(0x28, 0x06);                   /* N_FP=0000 0110                     */
  wr_reg(0x29, 0x06);                   /* P_BP=0000 0110                     */
  wr_reg(0x2A, 0x06);                   /* P_FP=0000 0110                     */
  wr_reg(0x2C, 0x06);                   /* I_BP=0000 0110                     */
  wr_reg(0x2D, 0x06);                   /* I_FP=0000 0110                     */
  wr_reg(0x3A, 0x01);                   /* N_RTN=0000, N_NW=001               */
  wr_reg(0x3B, 0x01);                   /* P_RTN=0000, P_NW=001               */
  wr_reg(0x3C, 0xF0);                   /* I_RTN=1111, I_NW=000               */
  wr_reg(0x3D, 0x00);                   /* DIV=00                             */
  wr_reg(0x3E, 0x38);                   /* SON=38h                            */
  wr_reg(0x40, 0x0F);                   /* GDON=0Fh                           */
  wr_reg(0x41, 0xF0);                   /* GDOF=F0h                           */

  /* Display ON Setting                                                       */
  wr_reg(0x90, 0x7F);                   /* SAP=0111 1111                      */
  wr_reg(0x26, 0x04);                   /* GON=0, DTE=0, D=01                 */
  delay_ms(100);
  wr_reg(0x26, 0x24);                   /* GON=1, DTE=0, D=01                 */
  wr_reg(0x26, 0x2C);                   /* GON=1, DTE=0, D=11                 */
  delay_ms(100);
  wr_reg(0x26, 0x3C);                   /* GON=1, DTE=1, D=11                 */

 #if (GLCD_SWAP_XY == 1)
  #if (GLCD_MIRROR_Y == 0)
   wr_reg (0x16, 0xA8);
  #else
   wr_reg (0x16, 0x68);
  #endif
 #else
  #if (GLCD_MIRROR_Y == 0)
   wr_reg (0x16, 0x08);
  #else
   wr_reg (0x16, 0xC8);
  #endif
 #endif

  /* Display scrolling settings ----------------------------------------------*/
  wr_reg(0x0E, 0x00);                   /* TFA MSB                            */
  wr_reg(0x0F, 0x00);                   /* TFA LSB                            */
  wr_reg(0x10, 320 >> 8);               /* VSA MSB                            */
  wr_reg(0x11, 320 &  0xFF);            /* VSA LSB                            */
  wr_reg(0x12, 0x00);                   /* BFA MSB                            */
  wr_reg(0x13, 0x00);                   /* BFA LSB                            */


  /* Turn backlight on with a certain backlight level                         */
  for (i = 0; i < 25 /* level */; i++) {
    PIOC->PIO_SODR = (LCD_BACKLIGHT);
    PIOC->PIO_SODR = (LCD_BACKLIGHT);
    PIOC->PIO_SODR = (LCD_BACKLIGHT);

    PIOC->PIO_CODR = (LCD_BACKLIGHT);
    PIOC->PIO_CODR = (LCD_BACKLIGHT);
    PIOC->PIO_CODR = (LCD_BACKLIGHT);
  }
  PIOC->PIO_SODR = (LCD_BACKLIGHT);

  return 0;
}

/**
  \fn          int32_t GLCD_Uninitialize (void)
  \brief       De-initialize Graphic LCD
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_Uninitialize (void) {

 return 0;
}

/**
  \fn          int32_t GLCD_SetForegroundColor (uint32_t color)
  \brief       Set foreground color
  \param[in]   color  Color value
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_SetForegroundColor (uint32_t color) {
  foreground_color = color;
  return 0;
}

/**
  \fn          int32_t GLCD_SetBackgroundColor (uint32_t color)
  \brief       Set background color
  \param[in]   color  Color value
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_SetBackgroundColor (uint32_t color) {
  background_color = color;
  return 0;
}

/**
  \fn          int32_t GLCD_ClearScreen (void)
  \brief       Clear screen (with active background color)
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_ClearScreen (void) {
  uint32_t i;

  GLCD_SetWindow(0, 0, GLCD_WIDTH, GLCD_HEIGHT);

  wr_cmd(0x22);
  wr_dat_start();
  for (i = 0; i < (GLCD_WIDTH * GLCD_HEIGHT); i++) {
    wr_dat_only(background_color);
  }
  wr_dat_stop();
  return 0;
}

/**
  \fn          int32_t GLCD_SetFont (GLCD_FONT *font)
  \brief       Set active font
  \param[in]   font   Pointer to font structure
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_SetFont (GLCD_FONT *font) {
  active_font = font;
  return 0;
}

/**
  \fn          int32_t GLCD_DrawPixel (uint32_t x, uint32_t y)
  \brief       Draw pixel (in active foreground color)
  \param[in]   x  x position in pixels (0 = left corner)
  \param[in]   y  y position in pixels (0 = upper corner)
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_DrawPixel (uint32_t x, uint32_t y) {

  wr_reg(0x02, x >>    8);            /* Column address start MSB           */
  wr_reg(0x03, x &  0xFF);            /* Column address start LSB           */
  wr_reg(0x04, x >>    8);            /* Column address end MSB             */
  wr_reg(0x05, x &  0xFF);            /* Column address end LSB             */

  wr_reg(0x06, y >>    8);            /* Row address start MSB              */
  wr_reg(0x07, y &  0xFF);            /* Row address start LSB              */
  wr_reg(0x08, y >>    8);            /* Row address end MSB                */
  wr_reg(0x09, y &  0xFF);            /* Row address end LSB                */

  wr_cmd(0x22);
  wr_dat(foreground_color);

  return 0;
}

/**
  \fn          int32_t GLCD_DrawHLine (uint32_t x, uint32_t y, uint32_t length)
  \brief       Draw horizontal line (in active foreground color)
  \param[in]   x      Start x position in pixels (0 = left corner)
  \param[in]   y      Start y position in pixels (0 = upper corner)
  \param[in]   length Line length
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_DrawHLine (uint32_t x, uint32_t y, uint32_t length) {
#if (GLCD_SWAP_XY == 0)
  y = (y + Scroll) % GLCD_HEIGHT;
#endif

  GLCD_SetWindow(x, y, length, 1);
  wr_cmd(0x22);
  wr_dat_start();

  while (length--) {
    wr_dat_only(foreground_color);
  }
  wr_dat_stop();
  return 0;
}

/**
  \fn          int32_t GLCD_DrawVLine (uint32_t x, uint32_t y, uint32_t length)
  \brief       Draw vertical line (in active foreground color)
  \param[in]   x      Start x position in pixels (0 = left corner)
  \param[in]   y      Start y position in pixels (0 = upper corner)
  \param[in]   length Line length in pixels
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_DrawVLine (uint32_t x, uint32_t y, uint32_t length) {
#if (GLCD_SWAP_XY == 0)
  y = (y + Scroll) % GLCD_HEIGHT;
#endif

  GLCD_SetWindow(x, y, 1, length);
  wr_cmd(0x22);
  wr_dat_start();

  while (length--) {
    wr_dat_only(foreground_color);
  }
  wr_dat_stop();
  return 0;
}

/**
  \fn          int32_t GLCD_DrawRectangle (uint32_t x, uint32_t y, uint32_t width, uint32_t height)
  \brief       Draw rectangle (in active foreground color)
  \param[in]   x      Start x position in pixels (0 = left corner)
  \param[in]   y      Start y position in pixels (0 = upper corner)
  \param[in]   width  Rectangle width in pixels
  \param[in]   height Rectangle height in pixels
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_DrawRectangle (uint32_t x, uint32_t y, uint32_t width, uint32_t height) {

  GLCD_DrawHLine (x,         y,          width);
  GLCD_DrawHLine (x,         y + height, width);
  GLCD_DrawVLine (x,         y,          height);
  GLCD_DrawVLine (x + width, y,          height);

  return 0;
}

/**
  \fn          int32_t GLCD_DrawChar (uint32_t x, uint32_t y, int32_t ch)
  \brief       Draw character (in active foreground color)
  \param[in]   x      x position in pixels (0 = left corner)
  \param[in]   y      y position in pixels (0 = upper corner)
  \param[in]   ch     Character
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_DrawChar (uint32_t x, uint32_t y, int32_t ch) {
  uint32_t i, j;
  uint32_t wb;
  uint8_t *ptr_ch_bmp;

#if (GLCD_SWAP_XY == 0)
  y = (y + Scroll) % GLCD_HEIGHT;
#endif

  if (active_font == NULL) return -1;

  ch        -= active_font->offset;
  wb         = (active_font->width + 7)/8;
  ptr_ch_bmp = (uint8_t *)active_font->bitmap + (ch * wb * active_font->height);

  GLCD_SetWindow(x, y, active_font->width, active_font->height);

  wr_cmd(0x22);
  wr_dat_start();

  for (i = 0; i < active_font->height; i++) {
    for (j = 0; j < active_font->width; j++) {
      wr_dat_only(((*ptr_ch_bmp >> (j & 7)) & 1) ? foreground_color : background_color);
      if (((j & 7) == 7) && (j != (active_font->width - 1))) ptr_ch_bmp++;
    }
    ptr_ch_bmp++;
  }
  wr_dat_stop();
  return 0;
}

/**
  \fn          int32_t GLCD_DrawString (uint32_t x, uint32_t y, const char *str)
  \brief       Draw string (in active foreground color)
  \param[in]   x      x position in pixels (0 = left corner)
  \param[in]   y      y position in pixels (0 = upper corner)
  \param[in]   str    Null-terminated String
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_DrawString (uint32_t x, uint32_t y, const char *str) {

  while (*str) { GLCD_DrawChar(x, y, *str++); x += active_font->width; }

  return 0;
}


/**
  \fn          int32_t GLCD_DrawBargraph (uint32_t x, uint32_t y, uint32_t width, uint32_t height, uint32_t val)
  \brief       Draw bargraph (in active foreground color)
  \param[in]   x      Start x position in pixels (0 = left corner)
  \param[in]   y      Start y position in pixels (0 = upper corner)
  \param[in]   width  Full bargraph width in pixels
  \param[in]   height Full bargraph height in pixels
  \param[in]   val    Active bargraph value (maximum value is 100)
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_DrawBargraph (uint32_t x, uint32_t y, uint32_t width, uint32_t height, uint32_t val) {
  uint32_t i, j;

  val = (val * width) / 100;            /* Scale value                        */

#if (GLCD_SWAP_XY == 0)
  y = (y + Scroll) % GLCD_HEIGHT;
#endif

  GLCD_SetWindow(x, y, width, height);

  wr_cmd(0x22);
  wr_dat_start();
  for (i = 0; i < height; i++) {
    for (j = 0; j < width; j++) {
      if(j >= val) {
        wr_dat_only(background_color);
      } else {
        wr_dat_only(foreground_color);
      }
    }
  }
  wr_dat_stop();

  return 0;
}

/**
  \fn          int32_t GLCD_DrawBitmap (uint32_t x, uint32_t y, uint32_t width, uint32_t height, const uint8_t *bitmap)
  \brief       Draw bitmap (bitmap from BMP file without header)
  \param[in]   x      Start x position in pixels (0 = left corner)
  \param[in]   y      Start y position in pixels (0 = upper corner)
  \param[in]   width  Bitmap width in pixels
  \param[in]   height Bitmap height in pixels
  \param[in]   bitmap Bitmap data
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_DrawBitmap (uint32_t x, uint32_t y, uint32_t width, uint32_t height, const uint8_t *bitmap) {
  int32_t   i, j;
  uint16_t *ptr_bmp;

  ptr_bmp = (uint16_t *)bitmap;

#if (GLCD_SWAP_XY == 0)
  y = (y + Scroll) % GLCD_HEIGHT;
#endif

  GLCD_SetWindow(x, y, width, height);

  wr_cmd(0x22);
  wr_dat_start();
  for (i = (height-1)*width; i > -1; i -= width) {
    for (j = 0; j < width; j++) {
      wr_dat_only(ptr_bmp[i+j]);
    }
  }
  wr_dat_stop();

  return 0;
}

/**
  \fn          int32_t GLCD_VScroll (uint32_t dy)
  \brief       Scroll picture on display vertically
  \param[in]   dy  Scroll size in pixels
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t GLCD_VScroll (uint32_t dy) {
#if (GLCD_SWAP_XY == 0)
  uint32_t i;

  Scroll = (Scroll + dy) % GLCD_HEIGHT;

  wr_reg(0x01, 0x08);
  wr_reg(0x14, Scroll >> 8);          /* VSP MSB */
  wr_reg(0x15, Scroll & 0xFF);        /* VSP LSB */

  /* Clear new space */
  GLCD_SetWindow(0, Scroll, GLCD_WIDTH, dy);

  wr_cmd(0x22);
  wr_dat_start();
  for(i = 0; i < (GLCD_WIDTH * dy); i++) {
    wr_dat_only(background_color);
  }
  wr_dat_stop();
  return (0);
#else
  return -1;
#endif
}

/**
  \fn          int32_t GLCD_FrameBufferAccess (bool enable)
  \brief       Enable or disable direct access to FrameBuffer
  \param[in]   enable Enable or disable request
                - true (1): enable
                - false(0): disable
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t  GLCD_FrameBufferAccess (bool enable) {
  GLCD_SetWindow(0, 0, GLCD_WIDTH, GLCD_HEIGHT);

  wr_reg (0x20, 0x0000);
  wr_reg (0x21, 0x0000);
  wr_cmd (0x22);
  return 0;
}

/**
  \fn          uint32_t GLCD_FrameBufferAddress (void)
  \brief       Retrieve FrameBuffer address
  \returns     FrameBuffer address
*/
uint32_t GLCD_FrameBufferAddress (void) {
  return (uint32_t)&LCD_DAT16;
}
